﻿using AbstractFoodOrdersContracts.BindingModels;
using AbstractFoodOrdersContracts.BusinessLogicsContracts;
using AbstractFoodOrdersContracts.SearchModels;
using AbstractFoodOrdersContracts.StoragesContracts;
using AbstractFoodOrdersContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractFoodOrdersBusinessLogic.BusinessLogics
{
	public class MessageInfoLogic: IMessageInfoLogic
	{
		private readonly ILogger _logger;
		private readonly IMessageInfoStorage _messageStorage;
		private readonly IClientLogic _clientLogic;

		public MessageInfoLogic(ILogger<MessageInfoLogic> logger, IMessageInfoStorage logic, IClientLogic clientLogic)
		{
			_logger = logger;
			_messageStorage = logic;
			_clientLogic = clientLogic;
		}

		public List<MessageInfoViewModel>? ReadList(MessageInfoSearchModel? model)
		{
			_logger.LogInformation("ReadList. MessageId:{MessageId}. ClientId:{ClientId}.", model?.MessageId, model?.ClientId);
			var list = model == null ? _messageStorage.GetFullList() : _messageStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}

		public bool Create(MessageInfoBindingModel model)
		{
			CheckModel(model);
			model.ClientId = _clientLogic.ReadElement(new ClientSearchModel() { Email = model.SenderName })?.Id;
			if (_messageStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}

		private void CheckModel(MessageInfoBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.MessageId))
			{
				throw new ArgumentNullException("Нет id письма", nameof(model.MessageId));
			}
			if (string.IsNullOrEmpty(model.SenderName))
			{
				throw new ArgumentNullException("Нет отправителя", nameof(model.SenderName));
			}
			if (string.IsNullOrEmpty(model.Subject))
			{
				throw new ArgumentNullException("Нет темы", nameof(model.Subject));
			}
			if (string.IsNullOrEmpty(model.Body))
			{
				throw new ArgumentNullException("Нет текста письма", nameof(model.Subject));
			}
			_logger.LogInformation("MessageInfo. MessageId:{MessageId}. SenderName:{SenderName}. Subject:{Subject}. Body:{Body}", model.MessageId, model.SenderName, model.Subject, model.Body);
		}
	}
}
