﻿using AbstractFoodOrdersContracts.BindingModels;
using AbstractFoodOrdersContracts.ViewModels;
using AbstractFoodOrdersDataModels.Enums;
using AbstractFoodOrdersDataModels.Models;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractFoodOrdersDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int DishId { get; private set; }
        [Required]
        public int ClientId { get; set; }
        public virtual Client Client { get; set; } = new();
		public int? ImplementerId { get; set; }
		public virtual Implementer? Implementer { get; set; } = new();
		[Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; }
        [Required]
        public DateTime DateCreate { get; private set; }
        public DateTime? DateImplement { get; private set; }
        public virtual Dish Dish { get; set; }
        public static Order? Create(AbstractFoodOrdersDatabase data, OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                DishId = model.DishId,
                ClientId = model.ClientId,
                Client = data.Clients.First(x => x.Id == model.ClientId),
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
				ImplementerId = model.ImplementerId,
				Implementer = model.ImplementerId.HasValue ? data.Implementers.First(x => x.Id == model.ImplementerId) : null,
			};
        }
        public void Update(AbstractFoodOrdersDatabase context, OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
			if (model.ImplementerId.HasValue) ImplementerId = model.ImplementerId;
			if (model.ImplementerId.HasValue) Implementer = context.Implementers.First(x => x.Id == model.ImplementerId);
			if (model.DateImplement.HasValue) DateImplement = model.DateImplement;
		}
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            DishId = DishId,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            ClientEmail = Client.Email,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
			ImplementerId = ImplementerId,
			ImplementerFIO = Implementer?.ImplementerFIO
		};
    }
}
