﻿using AbstractFoodOrdersContracts.BindingModels;
using AbstractFoodOrdersContracts.SearchModels;
using AbstractFoodOrdersContracts.StoragesContracts;
using AbstractFoodOrdersContracts.ViewModels;
using AbstractFoodOrdersListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractFoodOrdersListImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
		private readonly DataListSingleton _source;
		public OrderStorage()
		{
			_source = DataListSingleton.GetInstance();
		}
		public List<OrderViewModel> GetFullList()
		{
			var result = new List<OrderViewModel>();
			foreach (var order in _source.Orders)
			{
				result.Add(AccessDishStorage(order.GetViewModel));
			}
			return result;
		}

		public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
		{
			var result = new List<OrderViewModel>();
			if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.OrderStatus.HasValue)
			{
				return result;
			}
			if (model.ImplementerId.HasValue && model.OrderStatus.HasValue)
			{
				foreach (var order in _source.Orders)
				{
					if (order.Status == model.OrderStatus)
					{
						result.Add(AccessDishStorage(order.GetViewModel));
					}
				}
			}
			else if (!model.DateFrom.HasValue || !model.DateTo.HasValue)
			{
				foreach (var order in _source.Orders)
				{
					if (order.Id == model.Id)
					{
						result.Add(AccessDishStorage(order.GetViewModel));
					}
				}
			}
			else
			{
				foreach (var order in _source.Orders)
				{
					if (order.DateCreate >= model.DateFrom && order.DateCreate <= model.DateTo)
					{
						result.Add(AccessDishStorage(order.GetViewModel));
					}
				}
			}
			return result;
		}

		public OrderViewModel? GetElement(OrderSearchModel model)
		{
			if (!model.Id.HasValue && !model.ImplementerId.HasValue && !model.OrderStatus.HasValue)
			{
				return null;
			}
			if (!model.Id.HasValue)
			{
				foreach (var order in _source.Orders)
				{
					if (model.ImplementerId.HasValue && model.OrderStatus.HasValue
						&& order.ImplementerId == model.ImplementerId && order.Status == model.OrderStatus)
					{
						return AccessDishStorage(order.GetViewModel);
					}
				}
			}
			else
			{
				foreach (var order in _source.Orders)
				{
					if (model.Id.HasValue && order.Id == model.Id)
					{
						return AccessDishStorage(order.GetViewModel);
					}
				}
			}
			return null;
		}

		public OrderViewModel? Insert(OrderBindingModel model)
		{
			model.Id = 1;
			foreach (var order in _source.Orders)
			{
				if (model.Id <= order.Id)
				{
					model.Id = order.Id + 1;
				}
			}
			var newOrder = Order.Create(model);
			if (newOrder == null)
			{
				return null;
			}
			_source.Orders.Add(newOrder);
			return AccessDishStorage(newOrder.GetViewModel);
		}

		public OrderViewModel? Update(OrderBindingModel model)
		{
			foreach (var order in _source.Orders)
			{
				if (order.Id == model.Id)
				{
					order.Update(model);
					return AccessDishStorage(order.GetViewModel);
				}
			}
			return null;
		}

		public OrderViewModel? Delete(OrderBindingModel model)
		{
			for (int i = 0; i < _source.Orders.Count; ++i)
			{
				if (_source.Orders[i].Id == model.Id)
				{
					var element = _source.Orders[i];
					_source.Orders.RemoveAt(i);
					return AccessDishStorage(element.GetViewModel);
				}
			}
			return null;
		}

		public OrderViewModel AccessDishStorage(OrderViewModel model)
		{
			foreach (var pastry in _source.Dishes)
			{
				if (pastry.Id == model.DishId)
				{
					model.DishName = pastry.DishName;
					break;
				}
			}
			return model;
		}
	}
}
