﻿using AbstractFoodOrdersContracts.BindingModels;
using AbstractFoodOrdersContracts.SearchModels;
using AbstractFoodOrdersContracts.StoragesContracts;
using AbstractFoodOrdersContracts.ViewModels;
using AbstractFoodOrdersFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace AbstractFoodOrdersFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;

        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders.Select(x => x.GetViewModel)
                .Select(y => { y.DishName = source.Dishes.FirstOrDefault(x => x.Id == y?.DishId)?.DishName; return y; }).ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
			if (!model.Id.HasValue || !model.DateFrom.HasValue || !model.DateTo.HasValue || !model.OrderStatus.HasValue)
			{
				return new();
			}
            if (model.OrderStatus.HasValue)
            {
                return source.Orders.Where(x => x.ImplementerId == model.ImplementerId)
                .Select(x => x.GetViewModel)
                .Select(y => { y.DishName = source.Dishes.FirstOrDefault(x => x.Id == y?.DishId)?.DishName; return y; }).ToList();
            }
            if (!model.DateFrom.HasValue || !model.DateTo.HasValue)
            {
                return source.Orders.Where(x => x.Id == model.Id)
                .Select(x => x.GetViewModel)
                .Select(y => { y.DishName = source.Dishes.FirstOrDefault(x => x.Id == y?.DishId)?.DishName; return y; }).ToList();
            }
            else
            {
				return source.Orders.Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
			   .Select(x => x.GetViewModel)
			   .Select(y => { y.DishName = source.Dishes.FirstOrDefault(x => x.Id == y?.DishId)?.DishName; return y; }).ToList();
			}
		}

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
			if (!model.Id.HasValue && !model.ImplementerId.HasValue && !model.OrderStatus.HasValue)
			{
				return null;
			}
            if (!model.Id.HasValue)
            {
				return AccessDishStorage(source.Orders
			   .FirstOrDefault(x => model.ImplementerId.HasValue && model.OrderStatus.HasValue
			   && x.ImplementerId == model.ImplementerId && x.Status == model.OrderStatus)
			   ?.GetViewModel);
			}
			else
			{
				return AccessDishStorage(source.Orders
				.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
				?.GetViewModel);
			}
		}

        //для загрузки названий изделия в заказе
        private OrderViewModel AccessDishStorage(OrderViewModel model)
        {
			string? dishName = source.Dishes.FirstOrDefault(x => x.Id == model?.DishId)?.DishName;
			if (dishName != null) model.DishName = dishName;
			return model;
		}

        public OrderViewModel? Insert(OrderBindingModel model)
        {
			model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
			var newOrder = Order.Create(model);
			if (newOrder == null)
			{
				return null;
			}
			source.Orders.Add(newOrder);
			source.SaveOrders();
			return AccessDishStorage(newOrder.GetViewModel);
		}

        public OrderViewModel? Update(OrderBindingModel model)
        {
			var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
			if (order == null)
			{
				return null;
			}
			order.Update(model);
			source.SaveOrders();
			return AccessDishStorage(order.GetViewModel);
		}

        public OrderViewModel? Delete(OrderBindingModel model)
        {
			var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				source.Orders.Remove(element);
				source.SaveOrders();
				return AccessDishStorage(element.GetViewModel);
			}
			return null;
		}
    }
}
